@@ -1,16 +1,17 @@
 """Read/Write Matlab-compatible MAT files
 """
 
-# Copyright (c) 2013-2018 Erling Andersen, Haukeland University Hospital, Bergen, Norway
+# Copyright (c) 2013-2022 Erling Andersen, Haukeland University Hospital, Bergen, Norway
 
 import os.path
 import logging
 import numpy as np
 import scipy
 import scipy.io
-import imagedata.formats
-import imagedata.axis
-from imagedata.formats.abstractplugin import AbstractPlugin
+from . import NotImageError, input_order_to_dirname_str, WriteNotImplemented, shape_to_str, sort_on_to_str,\
+    SORT_ON_SLICE
+from ..axis import UniformLengthAxis
+from .abstractplugin import AbstractPlugin
 
 logger = logging.getLogger(__name__)
 
@@ -66,10 +67,10 @@ class MatPlugin(AbstractPlugin):
             self: format plugin instance
             f: file handle or filename (depending on self._need_local_file)
             opts: Input options (dict)
-            hdr: Header dict
+            hdr: Header
         Returns:
             Tuple of
-                hdr: Header dict
+                hdr: Header
                     Return values:
                         - info: Internal data for the plugin
                             None if the given file should not be included (e.g. raw file)
@@ -93,8 +94,8 @@ class MatPlugin(AbstractPlugin):
             logger.debug('matplugin._read_image variable {}'.format(name))
             si = self._reorder_to_dicom(mdict[name])
             logger.info("Data shape _read_image MAT: {} {}".format(si.shape, si.dtype))
-        except imagedata.formats.NotImageError:
-            raise imagedata.formats.NotImageError('{} does not look like a MAT file'.format(f))
+        except NotImageError:
+            raise NotImageError('{} does not look like a MAT file'.format(f))
         return info, si
 
     def _set_tags(self, image_list, hdr, si):
@@ -103,22 +104,22 @@ class MatPlugin(AbstractPlugin):
         Args:
             self: format plugin instance
             image_list: list with (info,img) tuples
-            hdr: Header dict
+            hdr: Header
             si: numpy array (multi-dimensional)
         Returns:
-            hdr: Header dict
+            hdr: Header
         """
 
         # Set spacing
-        hdr['spacing'] = (1.0, 1.0, 1.0)
+        hdr.spacing = (1.0, 1.0, 1.0)
 
         axes = list()
-        axes.append(imagedata.axis.UniformLengthAxis(
+        axes.append(UniformLengthAxis(
             'row',
             0,
             si.shape[-2])
         )
-        axes.append(imagedata.axis.UniformLengthAxis(
+        axes.append(UniformLengthAxis(
             'column',
             0,
             si.shape[-1])
@@ -127,19 +128,19 @@ class MatPlugin(AbstractPlugin):
         nt = nz = 1
         if si.ndim > 2:
             nz = si.shape[-3]
-            axes.insert(0, imagedata.axis.UniformLengthAxis(
+            axes.insert(0, UniformLengthAxis(
                 'slice',
                 0,
                 nz)
                         )
         if si.ndim > 3:
             nt = si.shape[-4]
-            axes.insert(0, imagedata.axis.UniformLengthAxis(
-                imagedata.formats.input_order_to_dirname_str(hdr['input_order']),
+            axes.insert(0, UniformLengthAxis(
+                input_order_to_dirname_str(hdr.input_order),
                 0,
                 nt)
                         )
-        hdr['axes'] = axes
+        hdr.axes = axes
         logger.debug('matplugin._set_tags nt {}, nz {}'.format(
             nt, nz))
         dt = 1
@@ -147,11 +148,11 @@ class MatPlugin(AbstractPlugin):
         tags = {}
         for slice in range(nz):
             tags[slice] = np.array(times)
-        hdr['tags'] = tags
+        hdr.tags = tags
         # logger.debug('matplugin._set_tags tags {}'.format(tags))
 
-        hdr['photometricInterpretation'] = 'MONOCHROME2'
-        hdr['color'] = False
+        hdr.photometricInterpretation = 'MONOCHROME2'
+        hdr.color = False
 
     def write_3d_numpy(self, si, destination, opts):
         """Write 3D numpy image as MAT file
@@ -168,7 +169,7 @@ class MatPlugin(AbstractPlugin):
         """
 
         if si.color:
-            raise imagedata.formats.WriteNotImplemented(
+            raise WriteNotImplemented(
                 "Writing color MAT images not implemented.")
 
         logger.debug('MatPlugin.write_3d_numpy: destination {}'.format(destination))
@@ -181,7 +182,7 @@ class MatPlugin(AbstractPlugin):
         self.spacing = si.spacing
         self.tags = si.tags
 
-        logger.info("Data shape write: {}".format(imagedata.formats.shape_to_str(si.shape)))
+        logger.info("Data shape write: {}".format(shape_to_str(si.shape)))
         if si.ndim == 4 and si.shape[0] == 1:
             si.shape = si.shape[1:]
         # if si.ndim == 2:
@@ -192,7 +193,7 @@ class MatPlugin(AbstractPlugin):
         #    raise ValueError("write_3d_series: slices of dicom template ({}) differ from input array ({}).".format(si.slices, slices))
 
         # newshape = tuple(reversed(si.shape))
-        # logger.info("Data shape matlab write: {}".format(imagedata.formats.shape_to_str(newshape)))
+        # logger.info("Data shape matlab write: {}".format(shape_to_str(newshape)))
         # logger.debug('matplugin.write_3d_numpy newshape {}'.format(newshape))
         # img = si.reshape(newshape, order='C')
         # img = si.reshape(newshape, order='F')
@@ -230,7 +231,7 @@ class MatPlugin(AbstractPlugin):
         """
 
         if si.color:
-            raise imagedata.formats.WriteNotImplemented(
+            raise WriteNotImplemented(
                 "Writing color MAT images not implemented.")
 
         logger.debug('MatPlugin.write_4d_numpy: destination {}'.format(destination))
@@ -244,7 +245,7 @@ class MatPlugin(AbstractPlugin):
         self.tags = si.tags
 
         # Defaults
-        self.output_sort = imagedata.formats.SORT_ON_SLICE
+        self.output_sort = SORT_ON_SLICE
         if 'output_sort' in opts:
             self.output_sort = opts['output_sort']
 
@@ -256,7 +257,7 @@ class MatPlugin(AbstractPlugin):
 
         logger.debug("write_4d_numpy: si dtype {}, shape {}, sort {}".format(
             si.dtype, si.shape,
-            imagedata.formats.sort_on_to_str(self.output_sort)))
+            sort_on_to_str(self.output_sort)))
 
         steps = si.shape[0]
         slices = si.shape[1]
