@@ -1,18 +1,19 @@
 """Read/Write Nifti-1 files
 """
 
-# Copyright (c) 2013-2018 Erling Andersen, Haukeland University Hospital, Bergen, Norway
+# Copyright (c) 2013-2022 Erling Andersen, Haukeland University Hospital, Bergen, Norway
 
 import os.path
 import tempfile
 import logging
 import math
-import numpy as np
-import imagedata.formats
-import imagedata.axis
-from imagedata.formats.abstractplugin import AbstractPlugin
 import nibabel
 import nibabel.spatialimages
+import numpy as np
+from . import NotImageError, WriteNotImplemented, input_order_to_dirname_str, shape_to_str, sort_on_to_str,\
+    SORT_ON_SLICE
+from ..axis import UniformLengthAxis
+from .abstractplugin import AbstractPlugin
 
 # import nitransforms
 
@@ -69,10 +70,10 @@ class NiftiPlugin(AbstractPlugin):
             self: format plugin instance
             f: file handle or filename (depending on self._need_local_file)
             opts: Input options (dict)
-            hdr: Header dict
+            hdr: Header
         Returns:
             Tuple of
-                hdr: Header dict
+                hdr: Header
                     Return values:
                         - info: Internal data for the plugin
                             None if the given file should not be included (e.g. raw file)
@@ -95,7 +96,7 @@ class NiftiPlugin(AbstractPlugin):
         try:
             img = nibabel.load(f)
         except nibabel.spatialimages.ImageFileError:
-            raise imagedata.formats.NotImageError(
+            raise NotImageError(
                 '{} does not look like a nifti file.'.format(f))
         except Exception:
             raise
@@ -123,10 +124,10 @@ class NiftiPlugin(AbstractPlugin):
         Args:
             self: format plugin instance
             image_list: list with (img,si) tuples
-            hdr: Header dict
+            hdr: Header
             si: numpy array (multi-dimensional)
         Returns:
-            hdr: Header dict
+            hdr: Header
         """
 
         img, si = image_list[0]
@@ -161,7 +162,7 @@ class NiftiPlugin(AbstractPlugin):
         elif _xyzt_units[1] == 'usec':
             dt = dt / 1000000.
         self.spacing = (float(dz), float(dy), float(dx))
-        hdr['spacing'] = (float(dz), float(dy), float(dx))
+        hdr.spacing = (float(dz), float(dy), float(dx))
 
         # Simplify shape
         self._reduce_shape(si)
@@ -173,7 +174,7 @@ class NiftiPlugin(AbstractPlugin):
             raise ValueError('qfac (pixdim[0]) should be 1 or -1')
 
         # Image orientation and positions
-        hdr['imagePositions'] = {}
+        hdr.imagePositions = {}
         if sform is not None and scode != 0:
             logger.debug("Method 3 - sform: orientation")
 
@@ -197,7 +198,7 @@ class NiftiPlugin(AbstractPlugin):
                     (Q[1, 2] * _slice + p[1]),
                     (Q[2, 2] * _slice + p[2])
                 ])
-                hdr['imagePositions'][_slice] = _p[::-1]
+                hdr.imagePositions[_slice] = _p[::-1]
 
         elif qform is not None and qcode != 0:
             logger.debug("Method 2 - qform: orientation")
@@ -224,7 +225,7 @@ class NiftiPlugin(AbstractPlugin):
                     ty * qfac * dz * _slice - qoffset_y,  # NIfTI is RAS+, DICOM is LPS+
                     tz * qfac * dz * _slice + qoffset_z
                 ])
-                hdr['imagePositions'][_slice] = _p[::-1]  # Reverse x,y,z
+                hdr.imagePositions[_slice] = _p[::-1]  # Reverse x,y,z
         else:
             logger.debug("Method 1 - assume axial: orientation")
             iop = np.array([0, 0, 1, 0, 1, 0])
@@ -234,8 +235,8 @@ class NiftiPlugin(AbstractPlugin):
                     0,  # NIfTI is RAS+, DICOM is LPS+
                     dz * _slice
                 ])
-                hdr['imagePositions'][_slice] = _p[::-1]  # Reverse x,y,z
-        hdr['orientation'] = iop
+                hdr.imagePositions[_slice] = _p[::-1]  # Reverse x,y,z
+        hdr.orientation = iop
 
         self.shape = si.shape
 
@@ -247,39 +248,48 @@ class NiftiPlugin(AbstractPlugin):
         tags = {}
         for z in range(nz):
             tags[z] = np.array(times)
-        hdr['tags'] = tags
+        hdr.tags = tags
 
         axes = list()
         if si.ndim > 3:
-            axes.append(imagedata.axis.UniformLengthAxis(
-                imagedata.formats.input_order_to_dirname_str(hdr['input_order']),
+            axes.append(UniformLengthAxis(
+                input_order_to_dirname_str(hdr.input_order),
                 0,
                 nt,
                 dt)
             )
         if si.ndim > 2:
-            axes.append(imagedata.axis.UniformLengthAxis(
+            axes.append(UniformLengthAxis(
                 'slice',
                 0,
                 nz,
                 dz)
             )
-        axes.append(imagedata.axis.UniformLengthAxis(
+        axes.append(UniformLengthAxis(
             'row',
             0,
             ny,
             dy)
         )
-        axes.append(imagedata.axis.UniformLengthAxis(
+        axes.append(UniformLengthAxis(
             'column',
             0,
             nx,
             dx)
         )
-        hdr['axes'] = axes
+        hdr.axes = axes
 
-        hdr['photometricInterpretation'] = 'MONOCHROME2'
-        hdr['color'] = False
+        hdr.photometricInterpretation = 'MONOCHROME2'
+        hdr.color = False
+
+        # Set dummy DicomHeaderDict
+        hdr.DicomHeaderDict = {}
+        for _slice in range(nz):
+            hdr.DicomHeaderDict[_slice] = []
+            for tag in range(nt):
+                hdr.DicomHeaderDict[_slice].append(
+                    (times[tag], None, hdr.empty_ds())
+                )
 
     # def nifti_to_affine(self, affine, shape):
     #
@@ -373,22 +383,22 @@ class NiftiPlugin(AbstractPlugin):
         Args:
             self: NiftiPlugin instance
             q: nifti Qform
-            hdr['spacing']
+            hdr.spacing
         Returns:
-            hdr: header dict
-                - hdr['imagePositions'][0]
-                - hdr['orientation']
-                - hdr['transformationMatrix']
+            hdr: header
+                - hdr.imagePositions[0]
+                - hdr.orientation
+                - hdr.transformationMatrix
         """
 
         # Swap back from nifti patient space, flip x and y directions
         affine = np.dot(np.diag([-1, -1, 1, 1]), q)
         # Set imagePositions for first slice
         x, y, z = affine[0:3, 3]
-        hdr['imagePositions'] = {0: np.array([z, y, x])}
-        logger.debug("getGeometryFromAffine: hdr imagePositions={}".format(hdr['imagePositions']))
+        hdr.imagePositions = {0: np.array([z, y, x])}
+        logger.debug("getGeometryFromAffine: hdr imagePositions={}".format(hdr.imagePositions))
         # Set slice orientation
-        ds, dr, dc = hdr['spacing']
+        ds, dr, dc = hdr.spacing
         logger.debug("getGeometryFromAffine: spacing ds {}, dr {}, dc {}".format(ds, dr, dc))
 
         colr = affine[:3, 0][::-1] / dr
@@ -401,7 +411,7 @@ class NiftiPlugin(AbstractPlugin):
         for i in range(3):
             orient.append(colr[i])
         logger.debug("getGeometryFromAffine: orient {}".format(orient))
-        hdr['orientation'] = orient
+        hdr.orientation = orient
         return
 
     # noinspection PyPep8Naming
@@ -662,7 +672,7 @@ class NiftiPlugin(AbstractPlugin):
     #
     #     # check if this is actually a volume and not all slices on the same location
     #     if np.linalg.norm(step) == 0.0:
-    #         raise imagedata.formats.NotImageError("Not a volume")
+    #         raise NotImageError("Not a volume")
     #
     #     affine = np.array(
     #         [[-image_orient1[0] * delta_c, -image_orient2[0] * delta_r, -step[0], -image_pos[0]],
@@ -690,7 +700,7 @@ class NiftiPlugin(AbstractPlugin):
         """
 
         if si.color:
-            raise imagedata.formats.WriteNotImplemented(
+            raise WriteNotImplemented(
                 "Writing color Nifti images not implemented.")
 
         logger.debug('NiftiPlugin.write_3d_numpy: destination {}'.format(destination))
@@ -709,7 +719,7 @@ class NiftiPlugin(AbstractPlugin):
         self.origin, self.orientation, self.normal = si.get_transformation_components_xyz()
         # slice_direction = _find_slice_direction(si, self.transformationMatrix, self.normal)
 
-        logger.info("Data shape write: {}".format(imagedata.formats.shape_to_str(si.shape)))
+        logger.info("Data shape write: {}".format(shape_to_str(si.shape)))
         assert si.ndim == 2 or si.ndim == 3, "write_3d_series: input dimension %d is not 3D." % si.ndim
 
         fsi = self._reorder_from_dicom(si, flip=False, flipud=True)
@@ -752,7 +762,7 @@ class NiftiPlugin(AbstractPlugin):
         """
 
         if si.color:
-            raise imagedata.formats.WriteNotImplemented(
+            raise WriteNotImplemented(
                 "Writing color Nifti images not implemented.")
 
         logger.debug('ITKPlugin.write_4d_numpy: destination {}'.format(destination))
@@ -770,7 +780,7 @@ class NiftiPlugin(AbstractPlugin):
         self.origin, self.orientation, self.normal = si.get_transformation_components_xyz()
 
         # Defaults
-        self.output_sort = imagedata.formats.SORT_ON_SLICE
+        self.output_sort = SORT_ON_SLICE
         if 'output_sort' in opts:
             self.output_sort = opts['output_sort']
 
@@ -784,7 +794,7 @@ class NiftiPlugin(AbstractPlugin):
 
         logger.debug("write_4d_numpy: si dtype {}, shape {}, sort {}".format(
             si.dtype, si.shape,
-            imagedata.formats.sort_on_to_str(self.output_sort)))
+            sort_on_to_str(self.output_sort)))
 
         steps = si.shape[0]
         slices = si.shape[1]
