@@ -19,11 +19,12 @@ import pydicom.dataset
 import pydicom.datadict
 # import numpy.core._multiarray_umath
 
-import imagedata.axis
-import imagedata.formats
-import imagedata.readdata as readdata
-import imagedata.formats.dicomlib.uid
-from imagedata.header import Header, add_template, add_geometry
+from .axis import UniformAxis, UniformLengthAxis, VariableAxis
+from .formats import INPUT_ORDER_NONE, INPUT_ORDER_TIME, INPUT_ORDER_B
+from .formats import input_order_to_dirname_str, shape_to_str, input_order_set, sort_on_set
+from .formats.dicomlib.uid import get_uid_for_storage_class
+from .readdata import read as r_read, write as r_write
+from .header import Header, deepcopy_DicomHeaderDict
 
 logger = logging.getLogger(__name__)
 
@@ -94,68 +95,71 @@ class Series(np.ndarray):
             # cls.__init_attributes(cls, obj)
             # obj.header = Header() # Already set in __array_finalize__
 
-            if issubclass(type(data), Series):
-                # Copy attributes from existing Series to newly created obj
-                # obj.__dict__ = data.__dict__.copy()  # carry forward attributes
-                obj.header = copy.copy(data.header)  # carry forward attributes
-
             # set the new 'input_order' attribute to the value passed
             obj.header.input_order = input_order
+
+            if issubclass(type(data), Series):
+                # Copy attributes from existing Series to newly created obj
+                obj.header = copy.copy(data.header)  # carry forward attributes
+                obj.input_order = data.input_order
+                obj.header.add_template(data.header)  # Includes DicomHeaderDict
+                obj.header.add_geometry(data.header, data.header)
+            else:
+                obj.header.set_default_values(obj.axes if axes is None else axes)
+
             # obj.header.set_default_values() # Already done in __array_finalize__
             if axes is not None:
                 obj.header.axes = copy.copy(axes)
-            add_template(obj.header, template)
-            add_geometry(obj.header, template, geometry)
+            obj.header.add_template(template)
+            obj.header.add_geometry(template, geometry)
             return obj
         logger.debug('Series.__new__: data is NOT subclass of Series, type {}'.format(type(data)))
 
-        if issubclass(type(data), np.uint16) or issubclass(type(data), np.float32):
-            obj = np.asarray(data).view(cls)
-            # cls.__init_attributes(cls, obj)
-            obj.header = Header()
-            obj.header.input_order = input_order
-            obj.header.set_default_values(obj.axes if axes is None else axes)
-            add_template(obj.header, template)
-            add_geometry(obj.header, template, geometry)
-            return obj
-
         # Assuming data is url to input data
         if isinstance(data, np.compat.basestring) or issubclass(type(data), PurePath):
             urls = data
         elif isinstance(data, list):
             urls = data
         else:
-            obj = np.asarray(data).view(cls)
+            if np.ndim(data) == 0:
+                obj = np.asarray([data]).view(cls)
+            else:
+                obj = np.asarray(data).view(cls)
             # cls.__init_attributes(cls, obj)
             obj.header = Header()
             obj.header.input_order = input_order
+            obj.header.input_format = type(data)
+            if np.ndim(data) == 0:
+                obj.header.axes = [UniformAxis('number', 0, 1)]
             obj.header.set_default_values(obj.axes if axes is None else axes)
-            add_template(obj.header, template)
-            add_geometry(obj.header, template, geometry)
+            obj.header.add_template(template)
+            obj.header.add_geometry(template, geometry)
             return obj
 
         # Read input, hdr is dict of attributes
-        hdr, si = readdata.read(urls, input_order, opts)
+        hdr, si = r_read(urls, input_order, opts)
 
         obj = np.asarray(si).view(cls)
         assert obj.header, "No Header found in obj.header"
 
         # set the new 'input_order' attribute to the value passed
         obj.header.input_order = input_order
+        obj.header.input_format = hdr.input_format
         # Copy attributes from hdr dict to newly created obj
         logger.debug('Series.__new__: Copy attributes from hdr dict to newly created obj')
         if axes is not None:
-            new_axes = axes
-        elif obj.axes is None and 'axes' in hdr:
-            new_axes = hdr['axes']
-        else:
-            new_axes = obj.axes
-        obj.header.set_default_values(new_axes)
-        for attr in hdr.keys():
-            setattr(obj.header, attr, hdr[attr])
+            obj.axes = copy.copy(axes)
+        elif hdr.axes is not None:
+            obj.axes = hdr.axes
+        obj.header.set_default_values(obj.axes)
+        obj.header.add_template(hdr)
+        obj.header.add_geometry(hdr, hdr)
+        # for attr in __attributes(hdr):
+        #     __set_attribute(obj.header, attr, __get_attribute(template, attr))
+        #     setattr(obj.header, attr, hdr[attr])
         # Store any template and geometry headers,
-        add_template(obj.header, template)
-        add_geometry(obj.header, template, geometry)
+        obj.header.add_template(template)
+        obj.header.add_geometry(template, geometry)
         # Finally, we must return the newly created object
         return obj
 
@@ -256,6 +260,8 @@ class Series(np.ndarray):
         if results and isinstance(results[0], Series):
             # logger.debug('Series.__array_ufunc__ add info to results:\n{}'.format(info))
             results[0].header = self._unify_headers(inputs)
+            results[0].setDicomAttribute('WindowCenter', results[0].max() / 2)
+            results[0].setDicomAttribute('WindowWidth', results[0].max())
 
         return results[0] if len(results) == 1 else results
 
@@ -277,10 +283,18 @@ class Series(np.ndarray):
         header = None
         for i, input_ in enumerate(inputs):
             if issubclass(type(input_), Series):
-                if header is None:
-                    header = input_.header
-                    # logger.debug('Series._unify_headers: copy header')
-                # else:
+                if input_.header is None:
+                    logger.warning('Series._unify_headers: new header')
+                    header = Header()
+                    header.input_order = INPUT_ORDER_NONE
+                else:
+                    logger.debug('Series._unify_headers: copy header')
+                    header = copy.copy(input_.header)
+                    header.input_order = input_.input_order
+                    header.set_default_values(input_.axes)
+                    header.add_template(input_.header)  # Includes DicomHeaderDict
+                    header.add_geometry(input_.header, input_.header)
+
                 # Here we could have compared the headers of
                 # the arguments and resolved discrepancies.
                 # The simplest resolution, however, is to take the
@@ -400,7 +414,7 @@ class Series(np.ndarray):
                     todo.append(('imagePositions', None))  # Wipe existing positions
                     if ipp is not None:
                         todo.append(('imagePositions', ipp))
-                elif axis.name == imagedata.formats.input_order_to_dirname_str(self.input_order):
+                elif axis.name == input_order_to_dirname_str(self.input_order):
                     # Select slice of tags
                     tags = self.__get_tags(spec)
                     todo.append(('tags', tags))
@@ -419,10 +433,14 @@ class Series(np.ndarray):
                 # Must copy the ret object before modifying. Otherwise, ret is a view to self.
                 ret.header = copy.copy(ret.header)
                 if ret.axes[-ret.ndim].name in ['slice', 'row', 'column', 'color']:
-                    ret.input_order = imagedata.formats.INPUT_ORDER_NONE
+                    ret.input_order = INPUT_ORDER_NONE
                 else:
                     raise IndexError('Unexpected axis {} after slicing'.format(ret.axes[0].name))
             _set_geometry(ret, todo)
+            new_uid = ret.header.new_uid()
+            ret.setDicomAttribute('SeriesInstanceUID', new_uid)
+            # ret.setDicomAttribute('SeriesInstanceUID', ret.header.new_uid())
+            ret.seriesInstanceUID = new_uid
         return ret
 
     def __get_sliceLocations(self, spec):
@@ -479,7 +497,7 @@ class Series(np.ndarray):
             start, stop, step, axis = specs[d]
             if axis.name == 'slice':
                 slice_spec = slice(start, stop, step)
-            elif axis.name == imagedata.formats.input_order_to_dirname_str(self.input_order):
+            elif axis.name == input_order_to_dirname_str(self.input_order):
                 tag_spec = slice(start, stop, step)
         # DicomHeaderDict[slice].tuple(tagvalue, filename, dicomheader)
         hdr = {}
@@ -527,8 +545,8 @@ class Series(np.ndarray):
             self.getDicomAttribute('StudyDate'), self.getDicomAttribute('StudyTime'),
             self.getDicomAttribute('SeriesDate'), self.getDicomAttribute('SeriesTime'),
             seriesNumber, seriesDescription,
-            imagedata.formats.shape_to_str(self.shape), self.dtype,
-            imagedata.formats.input_order_to_dirname_str(self.input_order))
+            shape_to_str(self.shape), self.dtype,
+            input_order_to_dirname_str(self.input_order))
 
     @staticmethod
     def __find_tag_in_hdr(hdr_list, find_tag):
@@ -550,7 +568,7 @@ class Series(np.ndarray):
             start, stop, step, axis = specs[d]
             if axis.name == "slice":
                 slice_spec = slice(start, stop, step)
-            elif axis.name == imagedata.formats.input_order_to_dirname_str(self.input_order):
+            elif axis.name == input_order_to_dirname_str(self.input_order):
                 tag_spec = slice(start, stop, step)
         # tags: dict[slice] is np.array(tags)
         new_tags = {}
@@ -579,7 +597,7 @@ class Series(np.ndarray):
         logger.debug('Series.write: url    : {}'.format(url))
         logger.debug('Series.write: formats: {}'.format(formats))
         logger.debug('Series.write: opts   : {}'.format(opts))
-        readdata.write(self, url, formats=formats, opts=opts)
+        r_write(self, url, formats=formats, opts=opts)
 
     @property
     def input_order(self):
@@ -603,7 +621,7 @@ class Series(np.ndarray):
 
     @input_order.setter
     def input_order(self, order):
-        if order in imagedata.formats.input_order_set:
+        if order in input_order_set:
             self.header.input_order = order
         else:
             raise ValueError("Unknown input order: {}".format(order))
@@ -643,7 +661,7 @@ class Series(np.ndarray):
 
     @input_sort.setter
     def input_sort(self, order):
-        if order is None or order in imagedata.formats.sort_on_set:
+        if order is None or order in sort_on_set:
             self.header.input_sort = order
         else:
             raise ValueError("Unknown sort order: {}".format(order))
@@ -670,7 +688,7 @@ class Series(np.ndarray):
 
     @sort_on.setter
     def sort_on(self, order):
-        if order in imagedata.formats.sort_on_set:
+        if order in sort_on_set:
             self.header.sort_on = order
         else:
             raise ValueError("Unknown sort order: {}".format(order))
@@ -897,7 +915,7 @@ class Series(np.ndarray):
         i = 0
         for d in super(Series, self).shape:
             self.header.axes.append(
-                imagedata.axis.UniformLengthAxis(
+                UniformLengthAxis(
                     _labels[i], 0, d, 1
                 )
             )
@@ -967,7 +985,7 @@ class Series(np.ndarray):
             >>> si.spacing = ds, dr, dc
         """
         try:
-            #if self.header.spacing is not None:
+            # if self.header.spacing is not None:
             #    return self.header.spacing
             slice_axis = self.find_axis('slice')
             ds = slice_axis.step
@@ -986,7 +1004,6 @@ class Series(np.ndarray):
     @spacing.setter
     def spacing(self, *args):
         if args[0] is None:
-            # self.header.spacing = None
             return
         logger.debug("spacing.setter {} {}".format(len(args), args))
         for arg in args:
@@ -1291,7 +1308,10 @@ class Series(np.ndarray):
             self.header.seriesInstanceUID = None
             return
         try:
-            self.header.seriesInstanceUID = str(uid)
+            if isinstance(uid, str):
+                self.header.seriesInstanceUID = uid
+            else:
+                self.header.seriesInstanceUID = str(uid)
         except AttributeError:
             raise TypeError("Given series instance UID is not printable")
 
@@ -1342,7 +1362,7 @@ class Series(np.ndarray):
             self.header.SOPClassUID = None
             return
         try:
-            self.header.SOPClassUID = imagedata.formats.dicomlib.uid.get_uid_for_storage_class(uid)
+            self.header.SOPClassUID = get_uid_for_storage_class(uid)
         except ValueError:
             raise
 
@@ -1656,7 +1676,7 @@ class Series(np.ndarray):
         Raises:
             ValueError: tags for dataset is not time tags
         """
-        if self.input_order == imagedata.formats.INPUT_ORDER_TIME:
+        if self.input_order == INPUT_ORDER_TIME:
             timeline = [0.0]
             for t in range(1, len(self.tags[0])):
                 timeline.append(self.tags[0][t] - self.tags[0][0])
@@ -1672,7 +1692,7 @@ class Series(np.ndarray):
         Raises:
             ValueError: tags for dataset is not b tags
         """
-        if self.input_order == imagedata.formats.INPUT_ORDER_B:
+        if self.input_order == INPUT_ORDER_B:
             return np.array(self.tags[0])
         else:
             raise ValueError("No b-value tags are available. Input order: {}".format(self.input_order))
@@ -1732,11 +1752,14 @@ class Series(np.ndarray):
         for s in slices:
             for t in tags:
                 tg, fname, im = self.DicomHeaderDict[s][t]
-                if _tag in im:
-                    im[_tag].value = value
-                else:
-                    VR = pydicom.datadict.dictionary_VR(_tag)
-                    im.add_new(_tag, VR, value)
+                # if _tag in im:
+                #     im[_tag].value = value
+                # else:
+                #     VR = pydicom.datadict.dictionary_VR(_tag)
+                #     im.add_new(_tag, VR, value)
+                # Always make a new attribute to avoid cross-talk after copying Series instances.
+                VR = pydicom.datadict.dictionary_VR(_tag)
+                im.add_new(_tag, VR, value)
 
     def getPositionForVoxel(self, r, transformation=None):
         """Get patient position for center of given voxel r
@@ -1803,12 +1826,20 @@ class Series(np.ndarray):
         # return int(r+0.5)[:3]
         return (r + 0.5).astype(int)[:3]
 
-    def to_rgb(self, colormap='Greys', lut=None):
+    def deepcopy(self):
+        """Create a copy using deepcopy."""
+        a = Series(np.copy(self), template=self, geometry=self)
+        a.header.DicomHeaderDict = deepcopy_DicomHeaderDict(self.header.DicomHeaderDict)
+        return a
+
+    def to_rgb(self, colormap='Greys_r', lut=None, norm='linear'):
         """Create an RGB color image of self.
 
         Args:
-            colormap (str): Matplotlib colormap name. Defaults: 'Greys'.
+            colormap (str): Matplotlib colormap name. Defaults: 'Greys_r'.
             lut (int): Number of rgb quantization levels. Default: None, lut is calculated from the voxel values.
+            norm (str or matplotlib.colors.Normalize): Normalization method. Either linear/log, or
+                the `.Normalize` instance used to scale scalar data to the [0, 1] range before mapping to colors using colormap.
 
         Returns:
             Series: RGB Series object
@@ -1817,52 +1848,53 @@ class Series(np.ndarray):
         if self.color:
             return self
 
-        # shape = self.shape + (3,)
-        if lut is None:
-            if np.issubdtype(self.dtype, np.floating):
-                lut = 256
-            else:
-                lut = (self.max().item()) + 1
-        # img = np.zeros(shape, dtype=np.uint8)
-        # if largest_image_pixel_value > 255:
-        #     scaling = 256 / largest_image_pixel_value
-        #     img[...,0] = np.uint8(self[:] * scaling)
-        #     img[...,1] = np.uint8(self[:] * scaling)
-        #     img[...,2] = np.uint8(self[:] * scaling)
-        # else:
-        #     img[...,0] = self[:]
-        #     img[...,1] = self[:]
-        #     img[...,2] = self[:]
-
+        import matplotlib as mpl
         import matplotlib.pyplot as plt
-        cm = plt.get_cmap(colormap, lut=lut)
-        if np.issubdtype(self.dtype, np.floating):
-            norm = (self - np.min(self)) / np.ptp(self)
+        from .viewer import get_window_level
+
+        if lut is None:
+            lut = 256 if self.dtype.kind == 'f' else (self.max().item()) + 1
+        if isinstance(norm, str):
+            if norm == 'linear':
+                norm = matplotlib.colors.Normalize
+            elif norm == 'log':
+                norm = matplotlib.colors.LogNorm
+            # elif norm == 'centered':
+            #     norm = matplotlib.colors.CenteredNorm
+            else:
+                raise ValueError('Unknown normalization function: {}'.format(norm))
+        if not issubclass(type(colormap), matplotlib.colors.Colormap):
+            colormap = plt.get_cmap(colormap, lut)
+        colormap.set_bad(color='k')  # Important for log display of non-positive values
+        colormap.set_under(color='k')
+        colormap.set_over(color='w')
+        if type(norm) == type:
+            window, level, vmin, vmax = get_window_level(self, norm, window=None, level=None)
+            norm = norm(vmin=vmin, vmax=vmax)
+        data = norm(self)
+        # if np.issubdtype(self.dtype, np.floating):
+        if self.dtype.kind == 'f':
             rgb = Series(
-                cm(norm, bytes=True)[...,:3],  # Strip off alpha color
+                colormap(data, bytes=True)[..., :3],  # Strip off alpha color
                 input_order=self.input_order,
                 geometry=self,
-                axes=self.axes + [imagedata.axis.VariableAxis('rgb',['r', 'g', 'b'])]
+                axes=self.axes + [VariableAxis('rgb', ['r', 'g', 'b'])]
             )
         else:
             rgb = Series(
-                cm(self, bytes=True)[...,:3],  # Strip off alpha color
+                colormap(data, bytes=True)[..., :3],  # Strip off alpha color
                 input_order=self.input_order,
                 geometry=self,
-                axes=self.axes + [imagedata.axis.VariableAxis('rgb',['r', 'g', 'b'])]
+                axes=self.axes + [VariableAxis('rgb', ['r', 'g', 'b'])]
             )
 
-        # rgb = Series(img, template=self, geometry=self)
-        #rgb = Series(img, input_order=self.input_order, geometry=self,
-        #             axes=self.axes + [imagedata.axis.VariableAxis('rgb',['r', 'g', 'b'])]
-        #             )
-        # rgb.axes = self.axes + [imagedata.axis.VariableAxis('rgb',['r', 'g', 'b'])]
         rgb.header.photometricInterpretation = 'RGB'
         rgb.header.color = True
-        add_template(rgb.header, self.header)
+        rgb.header.add_template(self.header)
         return rgb
 
-    def show(self, im2=None, fig=None, cmap='Greys', window=None, level=None, link=False):
+    def show(self, im2=None, fig=None, colormap='Greys_r', norm='linear', colorbar=None, window=None, level=None,
+             link=False):
         """Show image
 
         With ideas borrowed from Erlend Hodneland (2021).
@@ -1870,7 +1902,10 @@ class Series(np.ndarray):
         Args:
             im2 (Series or list of Series): Series or list of Series which will be displayed in addition to self.
             fig (matplotlib.plt.Figure, optional): if already exist
-            cmap (str): color map for display. Default: 'Greys'
+            colormap (str): color map for display. Default: 'Greys_r'
+            norm (str or matplotlib.colors.Normalize): Normalization method. Either linear/log, or
+                the `.Normalize` instance used to scale scalar data to the [0, 1] range before mapping to colors using colormap.
+            colorbar (bool): Display colorbar with image. Default: None: determine colorbar based on colormap and norm.
             window (number): window width of signal intensities. Default is DICOM Window Width.
             level (number): window level of signal intensities. Default is DICOM Window Center.
             link (bool): whether scrolling is linked between displayed images. Default: False
@@ -1879,7 +1914,7 @@ class Series(np.ndarray):
             ValueError: when image is not a subclass of Series, or too many viewports are requested.
             IndexError: when there is a mismatch with images and viewports.
         """
-        from imagedata.viewer import Viewer, default_layout
+        from .viewer import Viewer, default_layout
         import matplotlib.pyplot as plt
 
         # im2 can be single image or list of images
@@ -1898,7 +1933,7 @@ class Series(np.ndarray):
         axes = default_layout(fig, len(images))
         try:
             viewer = Viewer(images, fig=fig, ax=axes,
-                            cmap=cmap, window=window, level=level, link=link)
+                            colormap=colormap, norm=norm, colorbar=colorbar, window=window, level=level, link=link)
         except AssertionError:
             raise
         v = viewer.connect()
@@ -1906,8 +1941,8 @@ class Series(np.ndarray):
         plt.show()
         viewer.disconnect()
 
-    def get_roi(self, roi=None, color='r', follow=False, vertices=False, im2=None, fig=None, cmap='Greys',
-                window=None, level=None, link=False, single=False):
+    def get_roi(self, roi=None, color='r', follow=False, vertices=False, im2=None, fig=None, colormap='Greys_r',
+                norm='linear', colorbar=None, window=None, level=None, link=False, single=False):
         """Let user draw ROI on image
 
         Args:
@@ -1917,7 +1952,10 @@ class Series(np.ndarray):
             vertices (bool): Return both grid mask and dictionary of vertices. Default: False.
             im2 (Series or list of Series): Series or list of Series which will be displayed in addition to self.
             fig (matplotlib.plt.Figure, optional) if already exist
-            cmap (str): colour map for display. Default: 'Greys'
+            colormap (str): colour map for display. Default: 'Greys_r'
+            norm (str or matplotlib.colors.Normalize): Normalization method. Either linear/log, or
+                the `.Normalize` instance used to scale scalar data to the [0, 1] range before mapping to colors using colormap.
+            colorbar (bool): Display colorbar with image. Default: None: determine colorbar based on colormap and norm.
             window (number): window width of signal intensities. Default is DICOM Window Width.
             level (number): window level of signal intensities. Default is DICOM Window Center.
             link (bool): whether scrolling is linked between displayed objects. Default: False.
@@ -1934,7 +1972,7 @@ class Series(np.ndarray):
             ValueError: when image is not a subclass of Series, or too many viewports are requested.
             IndexError: when there is a mismatch with images and viewports.
         """
-        from imagedata.viewer import Viewer, default_layout, grid_from_roi
+        from .viewer import Viewer, default_layout, grid_from_roi
         import matplotlib.pyplot as plt
 
         # im2 can be single image or list of images
@@ -1953,7 +1991,7 @@ class Series(np.ndarray):
         axes = default_layout(fig, len(images))
         try:
             viewer = Viewer(images, fig=fig, ax=axes, follow=follow,
-                            cmap=cmap, window=window, level=level, link=link)
+                            colormap=colormap, norm=norm, colorbar=colorbar, window=window, level=level, link=link)
         except AssertionError:
             raise
         v = viewer.connect_draw(roi=roi, color=color)
@@ -1975,8 +2013,8 @@ class Series(np.ndarray):
                 new_grid = np.zeros((self.slices, self.rows, self.columns), dtype=np.ubyte)
         new_roi = Series(new_grid, input_order=input_order, template=self, geometry=self)
         new_roi.seriesDescription = 'ROI'
-        new_roi.setDicomAttribute('WindowCenter',.5)
-        new_roi.setDicomAttribute('WindowWidth',1)
+        new_roi.setDicomAttribute('WindowCenter', .5)
+        new_roi.setDicomAttribute('WindowWidth', 1)
         if vertices:
             return new_roi, viewer.get_roi()  # Return grid and vertices
         else:
