@@ -3,10 +3,9 @@
 Defines generic functions.
 """
 
-# Copyright (c) 2018 Erling Andersen, Haukeland University Hospital, Bergen, Norway
+# Copyright (c) 2018-2022 Erling Andersen, Haukeland University Hospital, Bergen, Norway
 
 from abc import ABCMeta, abstractmethod  # , abstractproperty
-# import imagedata.transports
 
 
 class NoOtherInstance(Exception):
@@ -24,6 +23,8 @@ class AbstractTransport(object, metaclass=ABCMeta):
     walk() method
     """
 
+    plugin_type = 'transport'
+
     def __init__(self, name, description, authors, version, url, schemes):
         object.__init__(self)
         self.__name = name
@@ -121,4 +122,4 @@ class AbstractTransport(object, metaclass=ABCMeta):
         Returns:
             description (str): Preferably a one-line string describing the object
         """
-        pass
\ No newline at end of file
+        pass
