@@ -26,6 +26,12 @@ class TestSeries(unittest.TestCase):
             'data/dicom/time/time00')
         r = si.__repr__()
 
+    #@unittest.skip("skipping test_str")
+    def test_str(self):
+        si = Series(
+            'data/dicom/time/time00/Image_00020.dcm')
+        r = '{}'.format(si)
+
     #@unittest.skip("skipping test_max")
     def test_max(self):
         si = Series(
@@ -46,6 +52,18 @@ class TestSeries(unittest.TestCase):
                 pydicom.datadict.tag_for_keyword('PatientID')),
             '19.02.07-14:04:17-STD-1.3.12.2.1107.5.2.43.66035')
 
+    #@unittest.skip("skipping test_create_series_1")
+    def test_create_series_1(self):
+        si = Series(np.uint16(1))
+        self.assertEqual(np.uint16, si.dtype)
+        self.assertEqual((1,), si.shape)
+
+    #@unittest.skip("skipping test_create_series_tuple_1D")
+    def test_create_series_tuple_1D(self):
+        si = Series((1, 2, 3))
+        self.assertEqual(np.int64, si.dtype)
+        self.assertEqual((3,), si.shape)
+
     #@unittest.skip("skipping test_create_series")
     def test_create_series(self):
         a = np.eye(128)
@@ -208,6 +226,12 @@ class TestSeries(unittest.TestCase):
         np.testing.assert_array_equal(a, s)
         self.assertEqual(s.slices, 4)
 
+    #@unittest.skip("skipping test_assign_slice_input_order")
+    def test_assign_slice_input_order(self):
+        si = Series('data/dicom/time', 'time')
+        sic = Series(si)
+        self.assertEqual(si.input_order, sic.input_order)
+
     #@unittest.skip("skipping test_slicing_z")
     def test_slicing_z(self):
         a1 = np.eye(128)
@@ -368,23 +392,59 @@ class TestSeries(unittest.TestCase):
 
     #@unittest.skip("skipping test_cross_talk")
     def test_cross_talk(self):
-        si = Series('data/dicom/time', 'time')
+        si = Series('data/dicom/time/time00')
+        # print('si before', si.getDicomAttribute('SeriesInstanceUID'), si.seriesInstanceUID)
         si1 = si[0]
         si1.seriesNumber = si.seriesNumber + 10
         self.assertNotEqual(si.seriesNumber, si1.seriesNumber)
+        # print('si after', si.getDicomAttribute('SeriesInstanceUID'), si.seriesInstanceUID)
+        # print('si1', si1.getDicomAttribute('SeriesInstanceUID'), si1.seriesInstanceUID)
+        self.assertNotEqual(si.seriesInstanceUID, si1.seriesInstanceUID)
 
-        si2 = Series('data/dicom/time', 'time')
+        si2 = Series('data/dicom/time/time00')
         si2.seriesNumber += 10
         self.assertNotEqual(si.seriesNumber, si2.seriesNumber)
 
-    # @nose.tools.raises(AssertionError)
-    @unittest.expectedFailure
+    #@unittest.skip("skipping test_cross_talk_wl_ref")
+    def test_cross_talk_wl_ref(self):
+        si = Series('data/dicom/time/time00')
+        si1 = si[0] * 10
+        self.assertNotEqual(si.getDicomAttribute('WindowWidth'), si1.getDicomAttribute('WindowWidth'))
+
+    #@unittest.skip("skipping test_cross_talk_wl")
+    def test_cross_talk_wl(self):
+        si = Series('data/dicom/time/time00')
+        si1 = si.deepcopy()[0] * 10
+        self.assertNotEqual(si.getDicomAttribute('WindowWidth'), si1.getDicomAttribute('WindowWidth'))
+
+    #@unittest.skip("skipping test_cross_talk_series_ref")
+    def test_cross_talk_series_ref(self):
+        si = Series('data/dicom/time/time00')
+        si1 = Series(si, input_order=si.input_order)
+        si1.setDicomAttribute('WindowWidth', 1)
+        self.assertNotEqual(si.getDicomAttribute('WindowWidth'), si1.getDicomAttribute('WindowWidth'))
+
+    #@unittest.skip("skipping test_cross_talk_series")
+    def test_cross_talk_series(self):
+        si = Series('data/dicom/time/time00')
+        si1 = si.deepcopy()
+        si1.setDicomAttribute('WindowWidth', 1)
+        self.assertEqual(1, si1.getDicomAttribute('WindowWidth'))
+        self.assertNotEqual(si.getDicomAttribute('WindowWidth'), si1.getDicomAttribute('WindowWidth'))
+
+    #@unittest.skip("skipping test_cross_talk_series_template")
+    def test_cross_talk_series_template(self):
+        si = Series('data/dicom/time/time00')
+        si1 = si.deepcopy()
+        si1.setDicomAttribute('WindowWidth', 1)
+        self.assertNotEqual(si.getDicomAttribute('WindowWidth'), si1.getDicomAttribute('WindowWidth'))
+
     #@unittest.skip("skipping test_cross_talk_spacing")
     def test_cross_talk_spacing(self):
         si = Series('data/dicom/time', 'time')
         si1 = si[0]
         si1.spacing = (1,1,1)
-        np.testing.assert_array_equal(si.spacing, si1.spacing)
+        self.assertNotEqual(si.spacing.tolist(), si1.spacing.tolist())
 
     #@unittest.skip("skipping test_cross_talk_2")
     def test_cross_talk_2(self):
@@ -414,6 +474,14 @@ class TestSeries(unittest.TestCase):
                      axes=si1.axes + [imagedata.axis.VariableAxis('rgb',['r', 'g', 'b'])]
                      )
 
+    #@unittest.skip("skipping test_header_axes")
+    def test_header_axes(self):
+        geometry = Series('data/dicom/time/time00')
+        si = Series(np.eye(128), geometry=geometry)
+        self.assertEqual(len(si.axes), 2)
+        for i in range(len(si.axes)):
+            self.assertEqual(len(si.axes[i]), si.shape[i])
+
 
 if __name__ == '__main__':
     unittest.main()
